;
; AutoIt Version: 3.0
; Language:       English
; Platform:       Win9x/NT
; Author:         Michael W. Bayley         
;
; Script Function:
;   Import Mozilla Firefox Bookmarks

#include <Constants.au3>
#include <GUIConstants.au3>

#include "TrayUrlDefs.au3"
#include "TrayUrlIniFunc.au3"

;
; Global values
;

; Title Information
$TitleBar = "TrayURL Firrefox Bookmarks Importer Version 2.0, (c) 2007 Michael W. Bayley"
$TitleBar2 = "TrayURL Firefox Bookmarks Importer"
$TitleBar3 = "Please select Bookmarks file to import"

; URL Entry Values
$Categories = 0
$Items = 0
$Index = 0
$Index2 = 0
$NotDone = 0
$Folder = ""
$Line = ""
$Depth = 0
$Cat = ""
$Name = ""
$Url = ""
$Text = ""
$Text2 = ""
$Counter = 0
$Perc = 0
$FullAction = ""
$FullActionDiscardAll = "DISCARDALL"
$FullActionDiscard = "DISCARD"
$FullActionRedirect = "REDIRECT"
$FullActionRedirectAll = "REDIRECTALL"

$CATMAX = $CatDefaultMax
$ITEMMAX = $UrlDefaultMax

; Microsoft Internet Explorer Favorites Values
$FavoritesExt = ".url"
$FavoritesSection = "InternetShortcut"
$FavoritesKey = "URL"

; Mozilla Firefox Bookmark Values
$BookmarksDefaultFile = "bookmarks.html"
$BookmarksFile = ""
$BookmarksDirs = 0
$BookmarksFiles = 0
$BookmarksOvers = 0
$BookmarksLargest = 0
$BookmarksList = ""
$BookmarksExported = 0

$BookmarksCat = "FF"
$BookmarksQuote = "&quot;"
$BookmarksTitle = "<!DOCTYPE NETSCAPE-Bookmark"
$BookmarksBegin = "<TITLE>Bookmarks</TITLE>"
$BookmarksFolder = "<DT><H3"
$BookmarksFolder2 = ">"
$BookmarksFolder3 = "</H3>"
$BookmarksStart = "<DL><p>"
$BookmarksEnd = "</DL><p>"
$BookmarksEntry = "<DT><A HREF="
$BookmarksEntry2 = Chr(34)
$BookmarksEntry3 = Chr(34)
$BookmarksEntry4 = ">"
$BookmarksEntry5 = "</A>"

; Dialog Values
$TextCat1 = "CATEGORY ALREADY EXISTS!"
$TextCat2 = "Current name:"
$TextCat3 = "New folder name:"
$TextCat4 = "Do you wish to rename the category or append the bookmarks to the existing category?"
$TextRename = "Rename"
$TextAppend = "Append"

$TextFull1a = "OUT OF CATEGORIES!"
$TextFull1b = "CATEGORY FULL!"
$TextSelect2 = "You can select the " & Chr(34) & "Export" & Chr(34) & " option "
$TextSelect2 = $TextSelect2 & "and send the URL entry to a Favorites folder named "
$TextSelect2 = $TextSelect2 & Chr(34) & $BookmarksCat & Chr(34) & "off of the program directory, "
$TextSelect2 = $TextSelect2 & "you can select " & Chr(34) & "Export All" & Chr(34) & " "
$TextSelect2 = $TextSelect2 & "send all remaining entries in that category to the special "
$TextSelect2 = $TextSelect2 & "Favorites folder, you can select " & Chr(34) & "Discard" & Chr(34)
$TextSelect2 = $TextSelect2 & " the current URL entry, you can select " & Chr(34) & "Discard All"
$TextSelect2 = $TextSelect2 & Chr(34) & " and discard the remaining URL entries in that category, "
$TextSelect2 = $TextSelect2 & "or you can edit the category name above and redirect the URL "
$TextSelect2 = $TextSelect2 & "entries to a new category."
$TextSelect3 = "You can select the " & Chr(34) & "Export" & Chr(34) & " option "
$TextSelect3 = $TextSelect3 & "and send the URL entry to a Favorites folder named "
$TextSelect3 = $TextSelect3 & Chr(34) & $BookmarksCat & Chr(34) & "off of the program directory, "
$TextSelect3 = $TextSelect3 & "you can select " & Chr(34) & "Export All" & Chr(34) & " "
$TextSelect3 = $TextSelect3 & "send all remaining entries in that category to the special "
$TextSelect3 = $TextSelect3 & "Favorites folder, you can select " & Chr(34) & "Discard" & Chr(34)
$TextSelect3 = $TextSelect3 & " the current URL entry, or you can select " & Chr(34) & "Discard All"
$TextSelect3 = $TextSelect3 & Chr(34) & " and discard the remaining URL entries in that category."
$TextFull2 = "Current name:"
$TextFull3 = "New folder name:"
$TextFull4 = "Do you wish to rename the category or append the bookmarks to the existing category?"
$TextNew = "New Category"
$TextExport = "Export"
$TextExportAll = "Export All"
$TextDiscard = "Discard"
$TextDiscardAll = "Discard All"

$TextUrl1 = "BOOKMARK ALREADY EXISTS!"
$TextUrl2 = "Bookmark folder:"
$TextUrl3 = "Current name:"
$TextUrl4 = "New name:"
$TextUrl5 = "Do you wish to rename the URL entry, overwrite it, export it to temporary location, or discard it?"
$TextOverwrite = "Overwrite"

$TextProgress = "Processing Mozilla Firefox Bookmarks..."
$FullDialogRetry = 0
$FullDialogRedirect = 1
$FullDialogRedirectAll = 2
$FullDialogDiscard = 3
$FullDialogDiscardAll = 4


;
; Main Code
;
; Entry:       <None>
; Return:      0 - No errors
;              1 - Fatal error encountered
; Procedure:   1. If command line parameter then message & exit.
;              2. If ini file not found, then create it.
;              3. Verify it is a TrayURL profile, error if not.
;              4. Get user to pick User Bookmark or Common Bookmarks.
;              5. Compute number of categories and files.
;              6. Import the Favorites.
;              7. Adjust menu size limits.
;              7. Exit all done.
;

; 1. If command line parameter then message & exit.

$msg = "This program will import Mozilla Firefox Bookmarks into TrayURL.  If no command line parameters are" & @CRLF
$msg = $msg & "supplied then the Bookmarks will be imported into the default profile, otherwise the profile specified"  & @CRLF
$msg = $msg & "on the command line will be used.  You will be given the opportunity to select the file to import the" & @CRLF
$msg = $msg & "Bookmarks from." & @CRLF & @CRLF
$msg = $msg & "The profile file must exist in the same directory as the program." & @CRLF & @CRLF
MsgBox(0, $TitleBar, $msg)

If $CmdLine[0] > 1 Then
   MsgBox(0, $TitleBar, "Error - Illegal number of command line arguments!")
   Exit(1)
ElseIf $CmdLine[0] == 1 Then
   $IniFile = @ScriptDir
   If StringRight($IniFile, 1) == "\" Then
      $IniFile = $IniFile & $CmdLine[1] & ".ini"
   Else
      $IniFile = $IniFile & "\" & $CmdLine[1] & ".ini"
   EndIf
EndIf


; 2. If ini file not found, then create it.
If FileExists($IniFile) <> 1 Then
   $msg = "Profile " & $IniFile & " not found!" & @CRLF & @CRLF
   $msg = $msg & "Would you like to create it?" & @CRLF & @CRLF
   If MsgBox(4, $TitleBar, $msg) <> 6 Then Exit(0)
   
   $msg = FileOpen($IniFile, 10)
   If $msg == -1 Then
      MsgBox(0, $TitleBar, "Fatal Error - Unable to create " & $IniFile & "!")
      Exit(1)
   EndIf
   FileClose($msg)

   IniWrite($IniFile, $IniProgram, $IniTag, $ProgramID)
   IniWrite($IniFile, $IniProgram, $IniMajor, $ProgramMajor)
   IniWrite($IniFile, $IniProgram, $IniMinor, $ProgramMinor)
   IniWrite($IniFile, $IniProgram, $IniBrowserType, $IniBrowserTypeDefault)
   IniWrite($IniFile, $IniCatSection, $IniItems, 0)
   IniWrite($IniFile, $IniCatMain, $IniItems, 0)
EndIf


; 3. Verify it is a TrayURL profile, error if not.
$Tag = IniRead($IniFile, $IniProgram, $IniTag, "")
$Major = Number(IniRead($IniFile, $IniProgram, $IniMajor, 0))
$Minor = Number(IniRead($IniFile, $IniProgram, $IniMinor, 0))

If $Tag <> $ProgramID Then
   MsgBox(0, $TitleBar, "Error - Profile " & $IniFile & " does not appear to be valid!")
   Exit(1)   
EndIf

If $Major <> $ProgramMajor OR $Minor <> $ProgramMinor Then
   $msg = "WARNING! The profile appears to be from a different version of TrayURL." & @CRLF
   $msg = $msg & "Proceeding may produce unknown results." & @CRLF & @CRLF
   $msg = $msg & "Proceed anyway?" & @CRLF & @CRLF
   If MsgBox(260, $TitleBar, $msg) <> 6 Then Exit(0)
EndIf


; 4. Get user to pick the Bookmarks file.
$BookmarksFile = BookmarksDialog()
If StringLen($BookmarksFile) == 0 Then Exit(0)

; Pick import to category.
$msg = "Please select a base category to import to (leave blank "
$msg &= "to import to " & $IniCatMain & "):"
While 1
   $BookmarksCat = InputBox($TitleBar2, $msg, $BookmarksCat, "", 300, 140)
   If @error Then Exit(0)
   If StringInStr($BookmarksCat, "[") OR StringInStr($BookmarksCat, "]") Then
      MsgBox(0, $TitleBar, "Error - Category may not contain '[' or ']' characters")
   Else
      ExitLoop
   EndIf
WEnd 


; 5. Compute number of categories and files.
$BookmarksDirs = 0
$BookmarksFiles = 0
$BookmarksOvers = 0
$BookmarksLargest = 0
$BookmarksList = ""
If BookmarksSearchStats($BookmarksFile, -1, 0, ".", $UrlDefaultMax) == -1 Then Exit(1)

$msg = "BOOKMARKS STATISTICS FOR " & Chr(34) & $BookmarksFile & Chr(34) & @CRLF & @CRLF
$msg = $msg & @TAB & "      Total Bookmark Folders = " & $BookmarksDirs & @CRLF
$msg = $msg & @TAB & "      Total Bookmarks Entries = " & $BookmarksFiles & @CRLF
$msg = $msg & @TAB & "      Total Oversized Folders = " & $BookmarksOvers & @CRLF
$msg = $msg & @TAB & "      Largest Folder (# of entries) = " & $BookmarksLargest & @CRLF & @CRLF

If $BookmarksOvers Then
   $msg = $msg & "WARNING!  There are " & $BookmarksOvers & " folder(s) with over the maximum number" & @CRLF
   $msg = $msg & "of bookmarks allowed.  These folders will be split into multiple" & @CRLF
   $msg = $msg & "categories as needed." & @CRLF & @CRLF
EndIf
$msg = $msg & "Proceed with the import?" & @CRLF
If MsgBox(4, $TitleBar2, $msg) <> 6 Then Exit(0)


; 6. Import the Bookmarks.
$Counter = 0
ProgressOn($titleBar2, "", $TextProgress, 20, 20)
While StringLen($BookmarksList)
   ; Get next directory.
   $pos = StringInStr($BookmarksList, Chr(1), 1, -2)
   If $pos Then
      $Folder = StringMid($BookmarksList, $pos + 1)
      $pos2 = StringInStr($Folder, Chr(1))
      $Line = Number(StringMid($Folder, $pos2 + 1))
      $Folder = StringLeft($Folder, $pos2 - 1)
      $BookmarksList = StringLeft($BookmarksList, $pos - 1)
   Else
      $Folder = $BookmarksList
      $pos2 = StringInStr($Folder, Chr(1))
      $Line = Number(StringMid($Folder, $pos2 + 1))
      $Folder = StringLeft($Folder, $pos2 - 1)
      $BookmarksList = ""
   EndIf

   ; Start the folder search.
   If $Folder = "." Then $Folder = ""
   $Text = DirAppend($BookmarksFolder, $Folder)
   $handle = FileOpen($BookmarksFile, 0)
   If $handle == -1 Then
      MsgBox(0, $TitleBar, "Error - Unable to open bookmarks file " & Chr(34) & $BookmarksFile & Chr(34) & ".")
      Exit(1)
   EndIf
   ; Build Category name.
   $Cat = DirAppend($BookmarksCat, $Folder)
   While 1
      $pos = StringInStr($Cat, "\")
      If $pos Then
         $Cat = StringLeft($Cat, $pos - 1) & "/" & StringMid($Cat, $pos + 1)
      Else
         ExitLoop
      EndIf
   WEnd
   While 1
      $pos = StringInStr($Cat, "[")
      If $pos Then
         $Cat = StringLeft($Cat, $pos - 1) & "{" & StringMid($Cat, $pos + 1)
      Else
         ExitLoop
      EndIf
   WEnd
   While 1
      $pos = StringInStr($Cat, "]")
      If $pos Then
         $Cat = StringLeft($Cat, $pos - 1) & "}" & StringMid($Cat, $pos + 1)
      Else
         ExitLoop
      EndIf
   WEnd

   ; Create the Category.
   $Cat = AddCategory($Cat)
   If $FullAction <> $FullActionDiscardAll AND $FullAction <> $FullActionRedirectAll Then
      $FullAction = ""
   EndIf

   ; Add the URL entries.
   $Depth = 0 
   While 1
      $Text = FileReadLine($handle, $Line)
      If StringInStr($Text, $BookmarksStart) Then
         $Depth += 1
      ElseIf StringInStr($Text, $BookmarksEnd) Then
         $Depth -= 1
         If $Depth == 0 Then ExitLoop
      ElseIf StringInStr($Text, $BookmarksEntry) Then
         If $Depth == 1 Then
            $pos = StringInStr($Text, $BookmarksEntry5)
            $Name = StringLeft($Text, $pos - 1)
            $pos = StringInStr($Name, $BookmarksEntry4, 0, -1)
            $Name = StringMid($Name, $pos + StringLen($BookmarksEntry4))

            While StringInStr($Name, $BookmarksQuote)
               $pos = StringInStr($Name, $BookmarksQuote)
               $Text2 = $Name
               $Name = StringLeft($Text2, $pos - 1)
               $Name &= Chr(34)
               $Name = StringMid($Text2, $pos + StringLen($BookmarksQuote))
            WEnd

            $pos = StringInStr($Text, $BookmarksEntry)
            $Url = StringMid($Text, $pos + StringLen($BookmarksEntry))
            $pos = StringInStr($Url, $BookmarksEntry2)
            $Url = StringMid($Url, $pos + StringLen($BookmarksEntry2))
            $pos = StringInStr($Url, $BookmarksEntry3)
            $Url = StringLeft($Url, $pos - 1)

            If StringLen($Name) AND StringLen($Url) Then
               $Counter += 1
               $Perc = ($Counter * 100) / $BookmarksFiles
               ProgressSet($Perc, "Category: " & $Cat, $TextProgress)
               $Cat = AddUrlEntry($Cat, $Name, $Url)
            EndIf
         EndIf
      EndIf
      $Line += 1
   WEnd
   FileClose($handle)
WEnd
ProgressOff()


; 7. Adjust menu size limits.
$Items = Number(IniRead($IniFile, $IniCatSection, $IniItems, 0))
$Categories = Number(IniRead($IniFile, $IniProgram, $IniCatMax, 0))
If $Items > $Categories Then
   If $Items > $CatDefaultMax Then $Items = $CatDefaultMax
   $Items = ($Items + 4) - Mod($Items + 4, 5)
   If IniWrite($IniFile, $IniProgram, $IniCatMax, $Items) <> 1 Then
      MsgBox(0, $TitleBar, "Error - Failure rewriting category maximum to INI file!")
   EndIf
EndIf

$Items = Number(IniRead($IniFile, $IniProgram, $IniUrlMax, 0))
If $BookmarksLargest > $Items Then
   If $BookmarksLargest > $UrlDefaultMax Then $BookmarksLargest = $UrlDefaultMax
   $BookmarksLargest = ($BookmarksLargest + 4) - Mod($BookmarksLargest + 4, 5)
   If IniWrite($IniFile, $IniProgram, $IniUrlMax, $BookmarksLargest) <> 1 Then
      MsgBox(0, $TitleBar, "Error - Failure rewriting URL entry maximum to INI file!")
   EndIf
EndIf 


; 8. Exit all done.
$msg = "Process Complete!" & @CRLF
If $BookmarksExported Then
   $msg = $msg & @CRLF
   $msg = $msg & "NOTICE: During the import process you elected to export one or more URL entries.  The" & @CRLF
   $msg = $msg & "entrie(s) have been saved in Microsoft Internet Explorer Favorites format in a directory" & @CRLF
   $msg = $msg & "named " & Chr(34) & $BookmarksCat & Chr(34) & "which was created here in the local program directory!" & @CRLF & @CRLF
EndIf

MsgBox(0, $TitleBar, $msg)
Exit(0)


;
; User Defined Functions
; Main Support Functions
;

;
; AddCategory($cat)
; Inputs:         $cat        Category to add
; Returns:        Category in use
;                 NULL if failure
; Procedure:      1. Skip if discarding.
;                 2. Skip if redirecting.
;                 3. Get user input if category exists.
;                 4. Add category if needed.
;                 5. If full then get user to decide on action.
;                 6. Return new category.
;
Func  AddCategory($cat)
   Local    $items, $index, $newcat, $cat2, $ret, $ret2

   ; 1. Skip if discarding.
   If $FullAction == $FullActionDiscardAll OR $FullAction == $FullActionDiscard Then
      Return($cat)
   EndIf


   ; 2. Skip if redirecting.
   If $FullAction == $FullActionRedirectAll OR $FullAction == $FullACtionRedirect Then
      Return($cat)
   EndIf


    ; 3. Get user input if category exists.
   If StringLen($cat) Then
      $items = Number(IniRead($IniFile, $IniCatSection, $IniItems, 0))
      $index = 1
      $newcat = $cat
      While $index <= $items
         $cat2 = IniRead($IniFile, $IniCatSection, $IniItem & $index, "")
         If StringLen($cat2) Then
            If $newcat == $cat2 Then
               $cat2 = CategoryDialog($newcat)
               If StringLen($cat2) == 0 Then
                  ExitLoop
               Else
                  $newcat = $cat2
                  $index = 0
               EndIf
            EndIf
         EndIf
         $index = $index + 1
      WEnd
   Else
      $newcat = $IniCatMain
      $cat2 = ""
   EndIf
   
   
   ; 4. Add category if needed.
   If StringLen($cat2) Then
      $cat2 = $newcat
      While 1
         $ret = AddCatToIni($TitleBar, $cat2, 0)

         ; 5. If full then get user to decide on action.
         If $ret == -1 Then
            $cats = IniRead($IniFile, $IniCatSection, $IniItems, 0)
            If $cats == $CATMAX Then
               $cat2 = CategoryFullDialog($cat2, 1)
               $ret2 = 1
            Else
               $cat2 = CategoryFullDialog($cat2, 0)
               $ret2 = 0
            EndIf
            
            If @error == $FullDialogRedirect Then
               ExitLoop
            ElseIf @error == $FullDialogRedirectAll Then
               If $ret2 Then
                  $FullAction = $FullActionRedirectAll
               Else
                  $FullAction = $FullActionRedirect
               EndIf
               ExitLoop
            ElseIf @error == $FullDialogDiscard Then
               ExitLoop
            ElseIf @error == $FullDialogDiscardAll Then
               If $ret2 Then
                  $FullAction = $FullActionDiscardAll
               Else
                  $FullAction = $FullActionDiscard
               EndIf
               ExitLoop
            EndIf
         EndIf
 
 
         If $ret == 1 Then
            $cat2 = $cat
            ExitLoop
         Else
            ExitLoop
         EndIf
      WEnd
      $newcat = $cat2
   EndIf


   ; 6. Return new category.
   Return($newcat)
EndFunc


;
; AddUrlEntry($cat, $name, $url)
; Inputs:         $cat        Category to add to
;                 $name       Entry name to add
;                 $url        URL to add
; Returns:        Category in use
;                 NULL if failure
; Procedure:      1. Skip if discarding.
;                 2. Write to Bookmark if redirecting.
;                 3. Attempt to add entry.
;                 4. If full then get user to decide on action.
;                 5. If abort overwrite then get user to select new category.
;                 6. Return category.
;
Func  AddUrlEntry($cat, $name, $url)
   Local    $ret, $ret2, $cats, $cat2, $warning

   ; 1. Skip if discarding.
   If $FullAction == $FullActionDiscardAll OR $FullAction == $FullActionDiscard Then
      Return($cat)
   EndIf


   ; 2. Write to bookmark if redirecting.
   If $FullAction == $FullActionRedirectAll OR $FullAction == $FullACtionRedirect Then
      $BookmarksExported = 1
      WriteFavorite($cat, $name, $url)
      Return($cat)
   EndIf


   ; 3. Attempt to add entry.
   $cat2 = $cat
   $warning = 1
   While 1
      $ret = AddEntryToIni($TitleBar, $cat2, $name, $url, "", "", 0, "", $warning)
      If $ret == 1 Then ExitLoop
      $warning = 1

      ; 4. If full then get user to decide on action.
      If $ret == -1 Then
         $cats = IniRead($IniFile, $IniCatSection, $IniItems, 0)
         If $cats == $CATMAX Then
            $cat2 = CategoryFullDialog($cat2, 1)
            $ret2 = 1
         Else
            $cat2 = CategoryFullDialog($cat2, 0)
            $ret2 = 0
         EndIf
            
         If @error == $FullDialogRetry Then
            $cat2 = AddCategory($cat2)
         ElseIf @error == $FullDialogRedirect Then
            $BookmarksExported = 1
            WriteFavorite($cat2, $name, $url)
            ExitLoop
         ElseIf @error == $FullDialogRedirectAll Then
            $BookmarksExported = 1
            WriteFavorite($cat2, $name, $url)
            If $ret2 Then
               $FullAction = $FullActionRedirectAll
            Else
               $FullAction = $FullActionRedirect
            EndIf
            ExitLoop
         ElseIf @error == $FullDialogDiscard Then
            ExitLoop
         ElseIf @error == $FullDialogDiscardAll Then
            If $ret2 Then
               $FullAction = $FullActionDiscardAll
            Else
               $FullAction = $FullActionDiscard
            EndIf
            ExitLoop
         EndIf
      EndIf


      ; 5. If abort overwrite then get user to select new category.
      If $ret == 0 Then
         $name2 = NameDialog($cat2, $name)

         If @error == $FullDialogRetry Then
            $warning = 0
            $name = $name2
         ElseIf @error == $FullDialogRedirect Then
            $BookmarksExported = 1
            WriteFavorite($cat2, $name, $url)
            ExitLoop
         ElseIf @error == $FullDialogDiscard Then
            ExitLoop
         EndIf
      EndIf
   WEnd


   ; 6. Return category. 
   Return($cat2)
EndFunc

;
; User Defined Functions
; Dialog Functions
;

;
; BookmarksDialog()
; Inputs:         <None>
; Returns:        Path to Bookmarks file to use
;                 NULL if user aborts
; Procedure:      1. Display dialog & get user response.
;                 2. Exit if abort.
;                 3. Verify the file is valid.
;                 4. Error if not
;                 5. Return data.
;
Func  BookmarksDialog()
   Local    $startDir, $bookmarksFile, $handle, $text
   Local    $valid, $notDone, $notDone2

   ; 1. Display dialog & get user response.
   If FileExists(DirAppend(@AppDataDir, "Mozilla\Firefox\Profiles")) == 1 Then
      $startDir = DirAppend(@AppDataDir, "Mozilla\Firefox\Profiles")
   ElseIf FileExists(DirAppend(@AppDataCommonDir, "Mozilla\Firefox\Profiles")) == 1 Then
      $startDir = DirAppend(@AppDataCommonDir, "Mozilla\Firefox\Profiles")
   Else
      $startDir = @AppDataDir
   EndIf
   
   $valid = 0
   $notDone = 1
   While $notDone
      $bookmarksFile = FileOpenDialog($TitleBar3, $startDir, "Bookmarks (*.html)", 3, $BookmarksDefaultFile)
   
   
      ; 2. Exit if abort.
      If @error Then Return("")
   

      ; 3. Verify the file is valid.
      $handle = FileOpen($bookmarksFile, 0)
      If $handle == -1 Then
         MsgBox(0, $TitleBar, "Error - Unable to open file " & $bookmarksFile & ".")
      Else
         $notDone2 = 1
         While $notDone2
            $text = FileReadLine($handle)
            If @error == -1 Then
               $notDone2 = 0
            ElseIf @error == 1 Then
               MsgBox(0, $TitleBar, "Error - Failure reading file " & $bookmarksFile & ".")
            ElseIf StringInStr($text, $BookmarksTitle) Then
               $valid = 1
               $notDone2 = 0
            EndIf
         WEnd
         FileClose($handle)


         ; 4. Error if not
         If $valid Then
            $notDone = 0
         Else
            MsgBox(0, $TitleBar2, "Error - " & $bookmarksFile & " does not appear to be a valid Firefox Bookmarks file.")
         EndIf
      EndIf
   WEnd


   ; 5. Return data.
   Return($bookmarksFile)
EndFunc


;
; CategoryDialog($cat)
; Inputs:         $cat     Category in conflict
; Returns:        Path to folder to use
;                 NULL if use same folder
; Procedure:      1. Display dialog
;                 2. Get user input. 
;                 3. Return the data.
;
Func  CategoryDialog($cat)
   Local    $dlg
   Local    $edit1, $edit2, $button1, $button2
   Local    $notDone, $msg, $newcat

   ; 1. Display dialog.
   $dlg = GUICreate($TitleBar2, 360, 218)
   GUICtrlCreateLabel($TextCat1, 105, 16, 260, 20)
   GUICtrlCreateLabel($TextCat2, 40, 50, 100, 20)
   $edit1 = GUICtrlCreateInput($cat, 135, 48, 180, 20)
   GUICtrlCreateLabel($TextCat3, 40, 80, 100, 20)
   $edit2 = GUICtrlCreateInput($cat, 135, 78, 180, 20)
   GUICtrlCreateLabel($TextCat4, 30, 120, 300, 40)
   $button1 = GUICtrlCreateButton($TextRename, 100, 168, 80, 30)
   $button2 = GUICtrlCreateButton($TextAppend, 200, 168, 80, 30)
   GUICtrlSetState($edit1, $GUI_DISABLE)
   GUICtrlSetState($button1, $GUI_DEFBUTTON)
   GUISetState(@SW_SHOW, $dlg)


   ; 2. Get user input.
   $notDone = 1
   While $notDone
      $msg = GUIGetMsg($dlg)
      Select
         Case $msg == $button1
            $newcat = GUICtrlRead($edit2)
            If StringInStr($newcat, "[") OR StringInStr($newcat, "]") Then
               MsgBox(0, $TitleBar, "Error - Category may not contain '[' or ']' characters")
               GUICtrlSetState($edit2, $GUI_FOCUS)
            Else
               $notDone = 0
            EndIf
         Case $msg == $button2
            $newcat = ""
            $notDone = 0
      EndSelect
   WEnd


   ; 3. Return the data.
   GUIDelete($dlg)
   Return($newcat)
EndFunc


;
; CategoryFullDialog($cat, $flag)
; Inputs:         $cat     Category in conflict
;                 $flag    Out of categories flag
; Returns:        New category
; Procedure:      1. Display dialog
;                 2. Get user input. 
;                 3. Return the data.
;
Func  CategoryFullDialog($cat, $flag)
   Local    $dlg
   Local    $edit1, $edit2, $button1, $button2, $button3, $button4, $button5
   Local    $notDone, $msg, $newcat, $ret

   ; 1. Display dialog.
   If $flag Then
      $dlg = GUICreate($TitleBar2, 370, 291)
      GUICtrlCreateLabel($TextFull1a, 130, 13, 120, 20)
      GUICtrlCreateLabel($TextSelect3, 30, 36, 310, 90)
   Else
      $dlg = GUICreate($TitleBar2, 370, 331)
      GUICtrlCreateLabel($TextFull1b, 135, 13, 120, 20)
      GUICtrlCreateLabel($TextSelect2, 30, 36, 310, 90)
   EndIf
   GUICtrlCreateLabel($TextCat2, 45, 140, 100, 20)
   $edit1 = GUICtrlCreateInput($cat, 140, 139, 180, 20)
   GUICtrlCreateLabel($TextCat3, 45, 170, 100, 20)
   $edit2 = GUICtrlCreateInput($cat, 140, 168, 180, 20)
   GUICtrlCreateLabel($TextCat4, 40, 205, 300, 40)
   If $flag  Then
      $button1 = -1
   Else
      $button1 = GUICtrlCreateButton($TextNew, 145, 290, 80, 30)
   EndIf
   $button2 = GUICtrlCreateButton($TextExport, 10, 245, 80, 30)
   $button3 = GUICtrlCreateButton($TextExportAll, 100, 245, 80, 30)
   $button4 = GUICtrlCreateButton($TextDiscard, 190, 245, 80, 30)
   $button5 = GUICtrlCreateButton($TextDiscardAll, 280, 245, 80, 30)
   GUICtrlSetState($edit1, $GUI_DISABLE)
   If $flag Then
      GUICtrlSetState($edit2, $GUI_DISABLE)
      GUICtrlSetState($button2, $GUI_DEFBUTTON)
   Else
      GUICtrlSetState($button1, $GUI_DEFBUTTON)
   EndIf
   GUISetState(@SW_SHOW, $dlg)


   ; 2. Get user input.
   $notDone = 1
   While $notDone
      $msg = GUIGetMsg($dlg)
      Select
         Case $msg == $button1
            $newcat = GUICtrlRead($edit2)
            If StringInStr($newcat, "[") OR StringInStr($newcat, "]") Then
               MsgBox(0, $TitleBar, "Error - Category may not contain '[' or ']' characters")
               GUISetState(@SW_ENABLE, $dlg)
               GUISetState(@SW_HIDE, $dlg)
               GUISetState(@SW_SHOW, $dlg)
               GUICtrlSetState($edit2, $GUI_FOCUS)
            ElseIf StringLen($newcat) Then
               $ret = $FullDialogRetry
               $notDone = 0
            Else
               GUISetState(@SW_DISABLED, $dlg)
               MsgBox(0, $TitleBar2, "Error - Blank category specified!")
               GUISetState(@SW_ENABLE, $dlg)
               GUISetState(@SW_HIDE, $dlg)
               GUISetState(@SW_SHOW, $dlg)
               GUICtrlSetState($edit2, $GUI_FOCUS)
            EndIf
         Case $msg == $button2
            $newcat = $cat
            $ret = $FullDialogRedirect
            $notDone = 0
         Case $msg == $button3
            $newcat = $cat
            $ret = $FullDialogRedirectAll
            $notDone = 0
         Case $msg == $button4
            $newcat = $cat
            $ret = $FullDialogDiscard
            $notDone = 0
         Case $msg == $button5
            $newcat = $cat
            $ret = $FullDialogDiscardAll
            $notDone = 0
      EndSelect
   WEnd


   ; 3. Return the data.
   GUIDelete($dlg)
   SetError($ret)
   Return($newcat)
EndFunc


;
; NameDialog($folder, $name)
; Inputs:         $folder  Folder containg URL in conflict
;                 $name    URL name in conflict
; Returns:        New name for Bookmark file to use
;                 $name for overwrite
;                 @error = action
; Procedure:      1. Display dialog
;                 2. Get user input. 
;                 3. Return the data.
;
Func  NameDialog($folder, $name)
   Local    $dlg
   Local    $edit1, $edit2, $edit3
   Local    $button1, $button2, $button3, $button4
   Local    $notDone, $msg, $newname, $ret

   ; 1. Display dialog.
   $dlg = GUICreate($TitleBar2, 370, 248)
   GUICtrlCreateLabel($TextUrl1, 113, 16, 260, 20)
   GUICtrlCreateLabel($TextUrl2, 35, 50, 100, 20)
   $edit1 = GUICtrlCreateInput($folder, 125, 48, 210, 20)
   GUICtrlCreateLabel($TextUrl3, 35, 80, 100, 20)
   $edit2 = GUICtrlCreateInput($name, 125, 78, 210, 20)
   GUICtrlCreateLabel($TextUrl4, 35, 110, 100, 20)
   $edit3 = GUICtrlCreateInput($name, 125, 108, 210, 20)
   GUICtrlCreateLabel($TextUrl5, 35, 150, 310, 40)
   $button1 = GUICtrlCreateButton($TextRename, 10, 198, 80, 30)
   $button2 = GUICtrlCreateButton($TextOverwrite, 100, 198, 80, 30)
   $button3 = GUICtrlCreateButton($TextExport, 190, 198, 80, 30)
   $button4 = GUICtrlCreateButton($TextDiscard, 280, 198, 80, 30)
   GUICtrlSetState($edit1, $GUI_DISABLE)
   GUICtrlSetState($edit2, $GUI_DISABLE)
   GUICtrlSetState($button1, $GUI_DEFBUTTON)
   GUISetState(@SW_SHOW, $dlg)


   ; 2. Get user input.
   $notDone = 1
   While $notDone
      $msg = GUIGetMsg($dlg)
      Select
         Case $msg == $button1
            ; Rename entry
            $newname = GUICtrlRead($edit3)
            If StringLen($newname) == 0 Then
               GUISetState(@SW_DISABLED, $dlg)
               MsgBox(0, $TitleBar2, "Error - Blank entry specified!")
               GUISetState(@SW_ENABLE, $dlg)
               GUISetState(@SW_HIDE, $dlg)
               GUISetState(@SW_SHOW, $dlg)
               GUICtrlSetState($edit3, $GUI_FOCUS)
            Else
               $ret = $FullDialogRetry
               $notDone = 0
            EndIf
         Case $msg == $button2
            ; Overwrite
            $newname = $name
            $ret = $FullDialogRetry
            $notDone = 0
         Case $msg == $button3
            ; Export
            $newname = GUICtrlRead($edit3)
            $ret = $FullDialogRedirect
            $notDone = 0
         Case $msg == $button4
            ; Discard
            $newname = $name
            $ret = $FullDialogDiscard
            $notDone = 0
      EndSelect
   WEnd


   ; 3. Return data.
   GUIDelete($dlg)
   SetError($ret)
   Return($newname)
EndFunc


;
; WriteFavorite($folder, $name, $url)
; Inputs:         $folder  Path to favorite folder
;                 $name    Filename of URL
;                 $url     URL to write
; Returns:        0        All ok
;                 1        Write failure
; Procedure:      1. Create the file, error if fail.
;                 2. Write the URL, report if error.
;                 3. Exit all done.
;
Func  WriteFavorite($folder, $name, $url)
      Local    $handle
      Local    $base, $pos

      ; 1. Create the file, report if fail.
      $base = DirAppend(@ScriptDir, $folder)
      While 1
         $pos = StringInStr($base, "/")
         If $pos Then
            $base = StringLeft($base, $pos - 1) & "\" & StringMid($base, $pos + 1)
         Else
            ExitLoop
         EndIf
      WEnd

      $handle = FileOpen(DirAppend($base, $name), 10)
      If $handle == -1 Then
         MsgBox(0, $TitleBar, "Error - Failure creating Favorite " & Chr(34) & $name & Chr(34) & " in category folder " & $cat & "!")
         Return(1)
      EndIf
      FileClose($handle)
      
      
      ; 2. Write the URL, report if failure.
      If IniWrite(DirAppend($base, $name), $FavoritesSection, $FavoritesKey, $url) <> 1 Then
         MsgBox(0, $TitleBar, "Error - Failure writing Favorite " & Chr(34) & $name & Chr(34) & " in category folder " & $cat & "!")
         Return(1)
      EndIf
   
   
   ; 3. Rexit all done.
   Return(0)
EndFunc


;
; User Defined Functions
; Search Functions
;

;
; BookmarksSearchStats($file, $handle, $line, $folder, $max)
;
; Description:    Search of $file for folders & bookmarks
; Inputs:         $file    Bookmarks file to search
;                 $handle  File handle to search
;                 $line    Current line number
;                 $folder  Current folder
;                 $max     Max if exceded increments over count
; Returns:        Current line if ok
;                 1 if error
;                 $BookmarksDirs incremented by 1 if dir contaisn files
;                 $BookmarksFiles incrmented by matching file count
;                 $BookmarksOvers incremnted if file count > max
;                 $BookmarksLargest replaced if file count is larger
;                 $BookmarksList appended dir name if files match
; Procedure:      1. Open the file, report if error.
;                 2. Get beginning of bookmarks.
;                 3. Perform the search.
;                 4. Return the data.
;
Func  BookmarksSearchStats($file, $handle, $line, $folder, $max)
   Local    $handle2, $line2, $line3, $folder2, $folder3
   Local    $notDone, $count, $text, $pos
   
   ; 1. Open the file, report if error.
   If $handle == -1 Then
      $handle2 = FileOpen($file, 0)
      If $handle2 == -1 Then
         MsgBox(0, $TitleBar, "Error - Unable to open bookmarks file " & Chr(34) & $file & Chr(34))
         Return(-1)
      EndIf

      ; 2. Get beginning of bookmarks.
      $line2 = 1
      $notDone = 1
      While $notDone
         $text = FileReadLine($handle2, $line2)
         If @error == -1 Then
            MsgBox(0, $TitleBar, "Error - Unable to locate beginning of bookmarks in " & Chr(34) & $file & Chr(34))
            FileClose($handle2)
            Return(-1)
         ElseIf @error == 1 Then
            MsgBox(0, $TitleBar, "Error - Failure reading bookmarks file " & Chr(34) & $file & Chr(34))
            FileClose($handle2)
            Return(-1)
         ElseIf StringInStr($text, $BookmarksBegin) Then
            ExitLoop
         Else
            $line2 += 1
         EndIf
      WEnd
   Else
      $handle2 = $handle
      $line2 = $line + 1
   EndIf


   ; 3. Perform the search.
   $count = 0
   $line3 = 0
   While 1
      $text = FileReadLine($handle2, $line2)
      If @error = -1 Then
         FileClose($handle2)
         ExitLoop
      ElseIf @error = 1 Then
         MsgBox(0, $TitleBar, "Error - Failure reading bookmarks file " & Chr(34) & $file & Chr(34))
         FileClose($handle2)
         Return(-1)
      ElseIf StringInStr($text, $BookmarksFolder) Then
         $folder2 = StringLeft($text, StringInStr($text, $BookmarksFolder3) - 1)
         $folder2 = StringMid($folder2, StringInStr($folder2, $BookmarksFolder2, 0, -1) + StringLen($BookmarksFolder2))

         While StringInStr($folder2, "/")
            $folder3 = $folder2
            $pos = StringInStr($folder3, "/")
            $folder2 = StringLeft($folder3, $pos - 1)
            $folder2 &= "-"
            $folder2 &= StringMid($folder3, $pos + 1)
         WEnd

         $folder3 = $folder & "/" & $folder2   
         $line2 = BookmarksSearchStats($file, $handle2, $line2, $folder3, $max)
         If $line2 == -1 Then Return(-1)
      ElseIf StringInStr($text, $BookmarksStart) Then
         $line3 = $line2
      ElseIf StringInStr($text, $BookmarksEnd) Then
         If $folder == "." Then FileClose($handle2)
         ExitLoop
      ElseIf StringInStr($text, $BookmarksEntry) Then
         $count += 1
      EndIf
      $line2 += 1
   WEnd


   ; 4. Return the data.
   If $count > $BookmarksLargest Then $BookmarksLargest = $count
   If $count > $max Then $BookmarksOvers = $BookmarksOvers + 1
   If $count Then
      $BookmarksDirs = $BookmarksDirs + 1
      $BookmarksFiles = $BookmarksFiles + $count
      If StringLen($BookmarksList) Then $BookmarksList = $BookmarksList & Chr(1)
      If StringLeft($folder, 2) == "./" Then
         $BookmarksList &= StringMid($folder, 3)
      Else
         $BookmarksList &= $folder
      EndIf
      $BookmarksList &= Chr(1) & $line3
   EndIf
   Return($line2)
EndFunc


;
; User Defined Functions
; Miscelaneous Functions
;

;
; DirAppend($dir, $file)
;
; Description:    Append the file string to the directory string.  Insert
;                 a "\" if needed.
; Inputs:         $dir     Directory string to append to
;                 $file    File string to append
; Procedure:      1. If directory is null then return file.
;                 2. If file is null then return directory.
;                 3. If directory ends in "\" and file starts with "\" then $dir + $file[2].
;                 4. If directory ends in "\" then $dir + $file.
;                 5. If file starts with "\" then $dir + $file.
;                 6. Else $dir + "\" + $file.
;
Func  DirAppend($dir, $file)
   ; 1. If directory is null then return file.
   If StringLen($dir) == 0 Then
      Return($file)
   EndIf


   ; 2. If file is null then return directory.
   If StringLen($file) == 0 Then
      Return($dir)
   EndIf


   ; 3. If directory ends in "\" and file starts with "\" then $dir + $file[2].
   If StringRight($dir, 1) == "\" AND StringLeft($file, 1) == "\" Then
      Return($dir & StringMid($file, 2))
   EndIf


   ; 4. If directory ends in "\" then $dir + $file.
   If StringRight($dir, 1) == "\" Then
      Return($dir & $file)
   EndIf

   
   ; 5. If file starts with "\" then $dir + $file.
   If StringLeft($file, 1) == "\" Then
      Return($dir & $file)
   EndIf

   
   ; 6. Else $dir + "\" + $file.
   Return($dir & "\" & $file)
EndFunc

