;
; AutoIt Version: 3.0
; Language:       English
; Platform:       Win9x/NT
; Author:         Michael W. Bayley
;
; Script Function:
;  Convert TrayURL 1.0 data file to 2.0   

#include "TrayUrlDefs.au3"
#include "TrayUrlIniFunc.au3"

;
; Global values
;

; Title Information
$TitleBar = "TrayURL Verion 1.0 to 2.0 Converter, (c)2007 Michael W. Bayley"
$Errors = 0

; Program Category & Item Values
$Catagories = 0
$Cat = ""
$Items = 0
$Index = 0
$Index2 = 0


;
; Main Code
;
; Entry:       <None>
; Return:      0 - No errors
;              1 - Fatal error encountered
; Procedure:   1. If command line parameter then message & exit.
;              3. Error if ini file not found.
;              3. Verify it is a TrayURL profile, error if not.
;              4, Fix existing keys and sections.
;              5, Add the new keys.
;              6. Update URL Main entries.
;              7. Update URL category entries.
;              8. Exit all done.
;

; 1. If command line parameter then message & exit.
If $CmdLine[0] > 1 Then
   $msg = "This program upgrades your TrayURL data file from version 1.0 to version 2.0." & @CRLF
   $msg = $msg & "This is required for the new version of TrayURL to recognize the profile file.  If" & @CRLF
   $msg = $msg & "you run the program with no command line arguments then the default profile" & @CRLF
   $msg = $msg & "will be used.  Otherwise you can specify a single profile on the command line." & @CRLF & @CRLF
   $msg = $msg & "In all instances the profile file msut exist in the same directory as the program." & @CRLF & @CRLF
   MsgBox(0, $TitleBar, $msg)
   Exit(1)
ElseIf $CmdLine[0] == 1 Then
   $IniFile = @ScriptDir
   If StringRight($IniFile, 1) == "\" Then
      $IniFile = $IniFile & $CmdLine[1] & ".ini"
   Else
      $IniFile = $IniFile & "\" & $CmdLine[1] & ".ini"
   EndIf
EndIf


; 2. Error if ini file not found.
If FileExists($IniFile) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Profile " & $IniFile & " not found!")
   Exit(1)   
EndIf


; 3. Verify it is a TrayURL profile, error if not.
$text = IniRead($IniFile, $IniProgram, $IniTag, "")
$Caterories = Number(IniRead($IniFile, $IniCatSection, $IniItems, 0))
$Items = Number(IniRead($IniFile, $IniCatMain, $IniItems, 0))

If $text == $ProgramID Then
   MsgBox(0, $TitleBar, "Notice - Profile " & $IniFile & " appears to already have been upgraded!")
   Exit(0)   
EndIf

If $cats == 0 AND $items == 0 Then
   MsgBox(0, $TitleBar, "Error - Profile " & $IniFile & " does not appear valid or contains no data!")
   Exit(1)   
EndIf 


; 4, Fix existing keys and sections.
If IniRenameSection($IniFile, "Catagories", $IniCatSection) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Failure writing to profile " & $IniFile & "!")
   $Errors += 1
EndIf

$text = IniRead($IniFile, $IniProgram, "WarnCatagoryDelete", 0)
If IniWrite($IniFile, $IniProgram, $IniWarnCatDelete, $text) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Failure writing to profile " & $IniFile & "!")
   $Errors += 1
EndIf
If IniDelete($IniFile, $IniProgram, "WarnCatagoryDelete") <> 1 Then
   MsgBox(0, $TitleBar, "Error - Failure deleting in profile " & $IniFile & "!")
   $Errors += 1
EndIf

$text = IniRead($IniFile, $IniProgram, "MaxCatagories", 0)
If IniWrite($IniFile, $IniProgram, $IniCatMax, $text) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Failure writing to profile " & $IniFile & "!")
   $Errors += 1
EndIf
If IniDelete($IniFile, $IniProgram, "MaxCatagories") <> 1 Then
   MsgBox(0, $TitleBar, "Error - Failure deleting in profile " & $IniFile & "!")
   $Errors += 1
EndIf


; 5, Add the new keys.
If IniWrite($IniFile, $IniProgram, $IniTag, $ProgramID) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Failure writing to profile " & $IniFile & "!")
   $Errors += 1
EndIf

If IniWrite($IniFile, $IniProgram, $IniMajor, $ProgramMajor) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Failure writing to profile " & $IniFile & "!")
   $Errors += 1
EndIf

If IniWrite($IniFile, $IniProgram, $IniMinor, $ProgramMinor) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Failure writing to profile " & $IniFile & "!")
   $Errors += 1
EndIf

If IniWrite($IniFile, $IniProgram, $IniPasterEnable, 0) <> 1 Then
   MsgBox(0, $TitleBar, "Error - Failure writing to profile " & $IniFile & "!")
   $Errors += 1
EndIf


; 6. Update URL Main entries.
$Cat = $IniCatMain
$Index = 1
While $Index <= $Items
   $record = ReadRecordByIndex($TitleBar, $Cat, $Index)
   If @error Then
      $record = CreateRecord("<BAD>", "", "", "", 0, "")
      $Errors += 1
   EndIf      
   $record[$RECLOGIN] = ""
   $record[$RECPASWD] = ""
   $record[$RECFLAGS] = 0
   $record[$RECDATE] = ""
   If WriteRecordByIndex($TitleBar, $Cat, $Index, $record) == 0 Then $Errors += 1
   $Index += 1
WEnd


; 7. Update URL category entries.
$Index = 1
While $Index <= $Categories
   $Cat = IniRead($IniFile, $IniCatSection, $Item & $Index, "")

   If StringLen($Cat) Then
      $Items = Number(IniRead($IniFile, $Cat, $Itema, 0))
      $Index2 = 1
      While $Index2 <= $Items
         $record = ReadRecordByIndex($TitleBar, $Cat, $Index2)
         If @error Then
            $record = CreateRecord("<BAD>", "", "", "", 0, "")
            $Errors += 1
         EndIf      
         $record[$RECLOGIN] = ""
         $record[$RECPASWD] = ""
         $record[$RECFLAGS] = 0
         $record[$RECDATE] = ""
         If WriteRecordByIndex($TitleBar, $Cat, $Index2, $record) == 0 Then $Errors += 1
         $Index2 += 1
      WEnd
   EndIf
   $Index += 1
WEnd


; 8. Exit all done.
MsgBox(0, $TitleBar, "Process Complete!  Errors = " & $Errors & @CRLF)
If $Errors Then Exit(1)
Exit(0)

